package pep;
import java.io.*;
import java.util.*;
import static pep.Main.nSubjects;
import static pep.Main.numPerVar;

/**This class opens each participant's dataset to calculate mean response
 * time and percentage error. Results are printed to Means.txt. This class
 * has a main method and can be run independently of the full model.*/

public class DataAnalysis{
 //----------VARIABLES--------------------------------------------------
 /**file name*/              static String fileName;
 /**number of variables*/    static int nVariables = numPerVar.length;
 //---------------------------------------------------------------------
 /**number of conditions*/   static int nConditions;
 /**condition number*/       static int condition;
 /**response time*/          static int rt;
 /**sum of response times*/  static int[] sumRT;
 /**number correct*/         static int[] nRT;
 /**mean response time*/     static int[] xRT;
 /**accuracy*/               static double accuracy;
 /**sum of accuracy scores*/ static double[] sumACC;
 /**number of responses*/    static int nACC[];
 /**percentage error*/       static double[] xERR;

 //----------Sequence Analysis Variables-------------------------------
 /**check sequence?*/        static boolean isCheckSeq = false;
 /**filter repetitions?*/    static boolean isTrimmed = false;
 /**check all rep types*/    static boolean isAllSeq = false;
 //--------------------------------------------------------------------
 /**distracter number*/      static int dist = 999;
 /**previous distracter*/    static int n1dist = 999;
 /**target number*/          static int targ = 999;
 /**previous target*/        static int n1targ = 999;
 /**congruency*/             static int cong = 999;
 /**previous congruency*/    static int n1cong = 999;
 /**threshold at response*/  static double thresh;

 /**Clears arrays and calls getMeans() for each participant.*/
 public static void main(String[] args){
  nConditions = 1;
  for(int k=0; k<nVariables; k++){
   nConditions *= (numPerVar[k]);
  }
  //double conditions if coding for n-1 congruency
  if(isCheckSeq) nConditions*=2;
  //test for the 15 possible feature repetition conditions
  if(isAllSeq) nConditions = 15;
  //this loop cycles through each participant
  for(int i=1; i<=nSubjects; i++){
   //clear arrays
   for(int v=0; v<nVariables; v++){
    sumRT = new int[nConditions];  sumACC = new double[nConditions];
    nRT = new int[nConditions];    nACC = new int[nConditions];
    xRT = new int[nConditions];    xERR = new double[nConditions];
   }
   //calculate participant means
   getMeans(i);
  }
 }


 /**Opens a participant datafile and calculates their condition means.
  * @param i Participant number, used for opening datafile.*/
 public static void getMeans(int i){
  BufferedReader in = null;
  FileOutputStream out = null;
  PrintStream p = null;
  StringTokenizer tokenizer = null;

  //open input and output streams
  try{
   fileName = "out"+i+".txt";
   in = new BufferedReader(new FileReader(fileName));
   if(i>1) out = new FileOutputStream("Means.txt", true);
   else out = new FileOutputStream("Means.txt");
   p = new PrintStream(out);
  }
  catch(Exception e){
   System.out.println("Error setting input and output streams.");
  }

  //read first trial
  try{
   tokenizer = new StringTokenizer(in.readLine());
  }
  catch(Exception e){
   System.out.println("No data in file.");
  }

  while(in!=null){
   try{
    condition=0;
    //find condition, rt, and accuracy
    //subject and response, not relevant for analysis
    tokenizer.nextToken();
    tokenizer.nextToken();
    //target
    targ = Integer.parseInt(tokenizer.nextToken());
    //distracters
    for(int j=0; j<Main.nDistracters.length; j++)
     dist = Integer.parseInt(tokenizer.nextToken());
    //determine conditions
    int orthogCount = 1;
    for(int n=0; n<nVariables; n++){
     condition += (Integer.parseInt(tokenizer.nextToken())*orthogCount);
     orthogCount *= (numPerVar[n]);
    }
    cong = condition;
    //add n-1 congruency
    if(n1cong<999 & isCheckSeq) condition += n1cong*orthogCount;
    //correct response
    tokenizer.nextToken();
    //response time
    rt = Integer.parseInt(tokenizer.nextToken());
    //accuracy
    accuracy = Integer.parseInt(tokenizer.nextToken());
    //threshold at response
    thresh = Double.parseDouble(tokenizer.nextToken());

    //-----Full Sequence Test-----
    if(isAllSeq){
     if(n1cong==1){
      if(cong==1){
       if(n1targ==dist){
        if(n1dist==targ)     condition = 14;
        else                 condition = 13;
       }
       else if(n1dist==targ) condition = 12;
       else if(n1targ==targ){
        if(n1dist==dist)     condition = 11;
        else                 condition = 10;
       }
       else if(n1dist==dist) condition = 9;
       else                  condition = 8;
      }
      else{
       if(n1targ==targ)     condition = 7;
       else if(n1dist==dist) condition = 6;
       else                  condition = 5;
      }
     }
     else{
      if(cong==1){
       if(n1targ==targ)     condition = 4;
       else if(n1dist==dist) condition = 3;
       else                  condition = 2;
      }
      else{
       if(n1targ==targ)     condition = 1;
       else                  condition = 0;
      }
     }
    }

    //-----Trimmed Data-----
    boolean isValid;
    if(!isTrimmed) isValid=true;
    else if(n1targ!=targ & n1dist!=dist){
     if(Main.isConflict){
      if(n1targ!=dist & n1dist!=targ) isValid=true;
      else isValid=false;
     }
     else isValid=true;
    }
    else isValid=false;
    if(isValid){
     //add on accuracy for the condition and increase counter
     sumACC[condition] += accuracy;
     nACC[condition]++;
     //add on rt and increases the counter for correct responses
     if(accuracy==1){
      sumRT[condition] += rt;
      nRT[condition]++;
     }
    }
    //----------------------

    n1dist = dist; n1targ = targ; n1cong = cong;

    //read next trial
    String tempString = in.readLine();
    //check for END_OF_FILE then send to tokenizer
    if(tempString.equals("END_OF_FILE")) in = null;
    else tokenizer = new StringTokenizer(tempString);
   }
   catch(NumberFormatException | IOException e3){
    in = null;
    System.out.println("No END_OF_FILE in data file.");
   }
  }

  //calculates then prints each mean rt
  for(int k=0; k<nConditions; k++){
   if(nRT[k]!=0) xRT[k] = sumRT[k]/nRT[k];
   else xRT[k] = 0;
   p.print(xRT[k]+"\t");
  }
  //calculates then prints each mean percentage error
  for(int k=0; k<nConditions; k++){
   if(nACC[k]!=0) xERR[k] = (1-(sumACC[k]/nACC[k]))*100;
   else xERR[k] = 0;
   p.print(xERR[k]+"\t");
  }
  p.print(thresh);
  //print new line
  p.println();

  //close input and output streams
  try{
   p.close(); out.close();
  }
  catch(Exception e2){
   System.out.println("Error closing input and output streams");
  }
 }
}