package pep;

/**This class defines the operations of Input nodes.*/
public class Input extends Node{
 /**stimulus bias*/              private double bias;
 /**stimulus signal*/            private double signal;
// /**predictiveness of stimulus*/ private double predict;
 
 /**Creates a new Input node.
  * @param subject Subject number used for static variable array.*/
 public Input(int subject){
  super(subject,RunTime.nEpisodes);
  threshold = .3;
  bias = 0;
  signal = 0;
 }

 /**Defines the input values for the node.
  * @param bs Amount of bias for the stimulus.
  * @param sig Amount of signal for the stimulus.*/
 public void turnOn(double bs,double sig){
  setBias(bs);
  signal = sig;
 }

 /**Computes a normally distributed bias value.
  * @param bs Bias multiplier.*/
 private void setBias(double bs){
  //average four random numbers to create a normal distribution
  bias = (java.lang.Math.random() + java.lang.Math.random());
  bias += (java.lang.Math.random() + java.lang.Math.random());
  bias /= 4;
  //multiply result by bias multiplier
  bias *= bs;
 }

 /**Calls the superclass method and activates Identity nodes.
  * @param stimNum The stimulus number.
  * @param isConflict Whether semantic conflict is present.*/
 public void activate(int stimNum,boolean isConflict){
  //set input to node and update activation
  setInput();
  activate();
  //targets and conflict distracters activate Identity nodes
  if(isConflict) actOut(stimNum);
  else if(isTarget) actOut(stimNum);
 }

 /**Updates bias and sets the input for cycle.*/
 private void setInput(){
  //adjust bias toward signal
  bias = (bias*.996) + (signal*java.lang.Math.random()*.004);
  //set input with bias and some noise
  input += (bias + java.lang.Math.random())/2;
 }

 /**Reduces signal and bias when stimulus removed from display.*/
 public void remove(){
  signal *= .98;
  bias *= .98;
 }

 /**Reduces signal and bias when stimulus is masked.*/
 public void mask(){
  signal = 0;
 }

 /**Shuts off signal and bias.*/
 public void shutOff(){
  signal = 0;
  bias = 0;
 }

 /**Sends activation to Identity node.
  * @param stimNum The stimulus number.*/
 public void actOut(int stimNum){
  if(activation>threshold){
   //calculate output
   double output = (activation-threshold)*java.lang.Math.random()*2;
   //increase output for targets
   if(isTarget) output*=3;
   output = range(output);
   //activate Identity nodes
   Main.runs[subj].identities[stimNum].input += output;
   //adjust total input for the change in input
   Identity.totInput[subj] += output;
  }
 }
}