package pep;

/**This superclass defines the basic fields and methods of Nodes*/
public class Node{
 /**rate of decay*/                   public double decay;
 /**incoming activation*/             public double input;
 /**current activation*/              public double activation;
 /**threshold to pass on activation*/ public double threshold;
 /**search strength*/                 public double strength;
 /**rate of connection building*/     public double writeConnect;
 /**rate of decay for connections*/   public double decayConnect;
 /**array of connections*/            public double[] connections;
 /**target truth value*/              public boolean isTarget;
 /**subject number*/                  public int subj;
 /**number of connections*/
 public static int[] nConnections = new int[Main.nSubjects];

 /**Creates a new Node.
  * @param subject Subject number used for static variable array.*/
 public Node(int subject){
  subj=subject;
  nConnections[subj] = 0;
  decay = .01;
  input = 0;
  activation = 0;
  isTarget = false;
 }

 /**Creates a new Node with connections to other nodes.
  * @param subject Subject number used for static variable array.
  * @param connectNum Number of possible connections to other nodes.*/
 public Node(int subject,int connectNum){
  this(subject);
  decayConnect = .0001;
  writeConnect = .01;
  connections = new double[connectNum];
 }

 /**Updates activation and searches memory.*/
 public void activate(){
  update();
  remember();
 }

 /**Searches memory and adjusts memory connections.*/
 public void remember(){
  //calculate output
  double output = range(activation-threshold)*strength;
  //pass output to memory
  if(Main.runs[subj].isStimuli && !isTarget){
   if(output>0) memSearch(output);
   memConnect(output);
  }
 } 

/**Uses input to update the activation of the Node.*/
 public void update(){
  //restrict input between 0 and 1
  input = range(input);
  //compute activation
  activation = activation*(1-decay) + input*decay;
  //restrict activation between 0 and 1
  activation = range(activation);
  //reset input
  input = 0;
 }

 /**Ranges activation between 0 and 1.
  * @param in Value to be ranged.
  * @return The newly ranged value.*/
 public double range(double in){
  if(in<0) return 0;
  else if(in>1) return 1;
  else return in;
 }

 /**Ranges activation between 0 and max.
  * @param in Value to be ranged.
  * @param max Maximum value to return.
  * @return The newly ranged value.*/
 public double range(double in,double max){
  if(in<0) return 0;
  else if(in>max) return max;
  else return in;
 }

 /**Activates episodes and weakens connections.
  * @param out (activation - threshold)*strength.*/
 public void memSearch(double out){
  int j =0;
  //first try larger loop
  int connectMinus10 = nConnections[subj] - 10;
  while(j<connectMinus10){
   //add input
   Main.runs[subj].memory[j].input += out*connections[j];
   //adjust connection
   connections[j] *= (1-(out*decayConnect)); j++;
   //repeat for nine more nodes
   Main.runs[subj].memory[j].input += out*connections[j];
   connections[j] *= (1-(out*decayConnect)); j++;
   Main.runs[subj].memory[j].input += out*connections[j];
   connections[j] *= (1-(out*decayConnect)); j++;
   Main.runs[subj].memory[j].input += out*connections[j];
   connections[j] *= (1-(out*decayConnect)); j++;
   Main.runs[subj].memory[j].input += out*connections[j];
   connections[j] *= (1-(out*decayConnect)); j++;
   Main.runs[subj].memory[j].input += out*connections[j];
   connections[j] *= (1-(out*decayConnect)); j++;
   Main.runs[subj].memory[j].input += out*connections[j];
   connections[j] *= (1-(out*decayConnect)); j++;
   Main.runs[subj].memory[j].input += out*connections[j];
   connections[j] *= (1-(out*decayConnect)); j++;
   Main.runs[subj].memory[j].input += out*connections[j];
   connections[j] *= (1-(out*decayConnect)); j++;
   Main.runs[subj].memory[j].input += out*connections[j];
   connections[j] *= (1-(out*decayConnect)); j++;
  }
  //run remaining connections
  for(; j<nConnections[subj]; j++){
   Main.runs[subj].memory[j].input += out*connections[j];
   connections[j] *= (1-(out*decayConnect));
  }
 }

 /**Builds a connection to new episode.
  * @param out (activation - threshold)*strength.*/
 public void memConnect(double out){
  //calculate incoming activation
  double temp = out*connections[nConnections[subj]];
  //add input to new Episode
  Main.runs[subj].memory[nConnections[subj]].input += temp;
  //build connection
  connections[nConnections[subj]] *= (1-writeConnect);
  connections[nConnections[subj]] += out*writeConnect;
 }

 /**Sets memory search strength for target.*/
 public void targeted(){
  //set memory connection strength
  strength = 2;
  //truth value used for several methods
  isTarget = true;
 }

 /**Sets memory search strength for distracter.
  * @param nSets The number of distracter sets.*/
 public void ignored(int nSets){
  //memory connection strength divided accross multiple distracting dimensions
  strength = 6/(double)nSets;
  //truth value used for several methods
  isTarget = false;
 }
}